import os
import re
import shutil
import sys

import click
import six

from great_expectations import DataContext
from great_expectations.cli.init_messages import (
    NEW_TEMPLATE_INSTALLED,
    NEW_TEMPLATE_PROMPT,
)

try:
    from termcolor import colored
except ImportError:
    colored = None


def cli_message(string):
    # the DOTALL flag means that `.` includes newlines for multiline comments inside these tags
    flags = re.DOTALL
    mod_string = re.sub(
        "<blue>(.*?)</blue>", colored("\g<1>", "blue"), string, flags=flags
    )
    mod_string = re.sub(
        "<cyan>(.*?)</cyan>", colored("\g<1>", "cyan"), mod_string, flags=flags
    )
    mod_string = re.sub(
        "<green>(.*?)</green>", colored("\g<1>", "green"), mod_string, flags=flags
    )
    mod_string = re.sub(
        "<yellow>(.*?)</yellow>", colored("\g<1>", "yellow"), mod_string, flags=flags
    )
    mod_string = re.sub(
        "<red>(.*?)</red>", colored("\g<1>", "red"), mod_string, flags=flags
    )

    six.print_(colored(mod_string))


def is_sane_slack_webhook(url):
    """Really basic sanity checking."""
    if url is None:
        return False

    return "https://hooks.slack.com/" in url.strip()



# This file is intended for tests whose functionality is a workaround for deficiencies in upstream libraries.
# In an ideal world, it is empty. As fixes come to be, we can replace its items.

import pybigquery
import sqlalchemy as sa

from great_expectations.dataset import SqlAlchemyDataset


def test_pybigquery_module_type_import(sqlitedb_engine):
    # We're really just testing that this particular hack works
    dataset = SqlAlchemyDataset(engine=sqlitedb_engine, custom_sql='SELECT "cat" as "pet_name"')
    dataset.engine.dialect = pybigquery.sqlalchemy_bigquery.BigQueryDialect()
    assert getattr(dataset._get_dialect_type_module(), "RECORD") == sa.types.JSON

    # assert getattr(pybigquery.sqlalchemy_bigquery._type_map, "RECORD") == sa.types.JSON



from .store import Store
from .validations_store import ValidationsStore
from .expectations_store import ExpectationsStore
from .html_site_store import HtmlSiteStore
from .metric_store import MetricStore, EvaluationParameterStore

from .store_backend import StoreBackend, InMemoryStoreBackend
from .tuple_store_backend import (
    TupleFilesystemStoreBackend,
    TupleS3StoreBackend,
    TupleGCSStoreBackend
)
from .database_store_backend import DatabaseStoreBackend
